"""
Views for ResumeManagerCore app.
"""

# System Imports.

# Third-Party Imports.
from django.db.models import Q
from django.contrib import messages
from django.contrib.auth import get_user_model
from django.contrib.auth.mixins import LoginRequiredMixin
from django.shortcuts import get_object_or_404, redirect
from django.views.generic import DetailView, ListView, UpdateView
from django.urls import reverse
from django.utils.safestring import mark_safe

# Internal Imports.
from .models import (
    Resume,
    ResumeSection,
    SectionCss,

    ContactInfo,
    EducationInstance,
    ExperienceInstance,
    ReferenceInstance,
    SkillCategory,
    SkillInstance,
    PillarCategory,
    PillarInstance,
)


class Index(LoginRequiredMixin, ListView):
    """Base view for ResumeManager index."""

    model = Resume
    template_name = 'resume_manager/index.html'
    context_object_name = 'resumes'

    def get_queryset(self):
        """Get "main" list of objects for template display."""

        # Return only resumes that are under example user or current user.
        example_user = get_user_model().objects.get(username='example_user')
        return Resume.objects.filter(user__in=[example_user, self.request.user])


# class DisplayResume(LoginRequiredMixin, DetailView):
class DisplayResume(DetailView):
    """Resume display view."""

    model = Resume
    template_name = 'resume_manager/display.html'

    def dispatch(self, request, *args, **kwargs):
        """
        Normally dispatch is left alone. However, here we override to make sure the user has
        permission to access resume before proceeding.

        If true, we call the original, inherited dispatch that proceeds to call the rest of the class logic.
        If false, we instead redirect to the resume index page for user.
        """

        # Pull url kwarg data.
        return_redirect = False
        self.resume_pk = kwargs['pk']

        # Check if resume with pk exists.
        try:
            self.resume = Resume.objects.get(pk=self.resume_pk)
        except Resume.DoesNotExist:
            # Failed to find corresponding resume with pk from url. Fallback to index.
            return_redirect = True
            messages.warning(request, 'Invalid resume. Redirecting to index.')

        if not return_redirect:
            # Resume was valid. Now ensure user is meant to access it.
            example_user = get_user_model().objects.get(username='example_user')
            if not (self.resume.user == example_user or self.resume.user == self.request.user):
                # Resume exists, but user doesn't have permission to access. Fallback to index.
                return_redirect = True
                messages.warning(request, 'Invalid resume. Redirecting to index.')

        # Handle if redirect.
        if return_redirect:
            return redirect(reverse('resume_manager_core:index'))

        # If we made it this far, then url was valid for user.
        # Proceed to rest of view logic.
        return super().dispatch(request, *args, **kwargs)

    def get_context_data(self, **kwargs):
        """Add additional context (variables) for template to display."""

        # Call parent logic.
        context = super().get_context_data(**kwargs)

        # Get general resume data.
        user_name = '{0} {1}'.format(self.resume.user.first_name, self.resume.user.last_name)
        user_profession = self.resume.profession.split('<br>')
        if isinstance(user_profession, list) and len(user_profession) > 1:
            user_profession_1 = user_profession.pop(0)
            user_profession_2 = '<br>'.join(user_profession)
        else:
            user_profession_1 = '<br>'.join(user_profession)
            user_profession_2 = None
        user_address = self.resume.user.profile.address

        # Get all sections for resume.
        resume_sections = ResumeSection.objects.filter(resume=self.resume)

        # Organize resume meta-data into parse-able structures.
        resume_data = []
        resume_pages = resume_sections.filter(parent_section=None)
        page_css_data = SectionCss.objects.filter(id=1)

        # Add each page as its own unique base element in dataset.
        for resume_page in resume_pages:

            # Get all related children.
            section_children = resume_sections.filter(
                parent_section=resume_page
            )

            # Build sub-dictionaries for children.
            section_children_data = []
            for child in section_children:
                section_children_data.append(self.build_resume_data(child, resume_sections))

            # Get all css styles associated with section.
            # Note: We always associate the base "page" styles here, with the full "page" sections.
            section_css_data = set()
            section_css = resume_page.css.all()
            section_css = section_css.union(page_css_data)  # Concatenate pulled css and "page" css.
            for css_object in section_css:
                css_classes = css_object.css_classes.strip().split(' ')
                for css_class in css_classes:
                    section_css_data.add(css_class)
            section_css_data = ' '.join(list(section_css_data))

            # Organize data value.
            # Note: We always associate the base "page" styles with the outer-most pages.
            resume_data.append({
                'section': resume_page,
                'section_title': None,
                'section_description': None,
                'section_children': section_children_data,
                'section_css': section_css_data,
                'separator_before': False,
                'separator_after': False,
                'display_category': 'None',
            })

        # Organize resume data into parse-able structures.
        # Handle "skills" section, if present.
        skill_categories = SkillCategory.objects.filter(resume=self.resume)
        skills = SkillInstance.objects.filter(category__in=skill_categories)
        skill_data = []
        if len(skills) > 0:
            for category in skill_categories:

                category_data = {
                    'category': category.name,
                    'skills': [],
                }

                # Add all skills that match category.
                for skill in skills:
                    if skill.category == category:
                        # Add skill to category.
                        category_data['skills'].append(skill)

                skill_data.append(category_data)

        # Handle "pillars" section, if present.
        pillar_categories = PillarCategory.objects.filter(resume=self.resume)
        pillars = PillarInstance.objects.filter(category__in=pillar_categories)
        pillar_data = []
        if len(pillars) > 0:
            for category in pillar_categories:

                category_data = {
                    'category': category.name,
                    'pillars': [],
                }

                # Add all pillars that match category.
                for pillar in pillars:
                    if pillar.category == category:
                        # Add pillar to category.
                        category_data['pillars'].append(pillar.text)

                pillar_data.append(category_data)

        # Pull together possible resume content.
        resume_content = {
            'name': user_name,
            'profession1': user_profession_1,
            'profession2': user_profession_2,
            'address': user_address,
            'contact': ContactInfo.objects.get(resume=self.resume),
            'education': EducationInstance.objects.filter(resume=self.resume) or None,
            'experience': ExperienceInstance.objects.filter(resume=self.resume) or None,
            'skills': skill_data,
            'references': ReferenceInstance.objects.filter(resume=self.resume) or None,
            'pillars': pillar_data,
        }

        # Update template context.
        context.update({
            'resume': self.resume,
            'resume_theme': self.resume.theme,
            'resume_data': resume_data,
            'resume_content': resume_content,
        })

        # Return updated context.
        return context

    def build_resume_data(self, current_section, all_sections):
        """Recursive function to parse meta-data and format structure for template output."""
        # Get all related children.
        section_children = all_sections.filter(
            parent_section=current_section
        )

        # Build sub-dictionaries for children.
        section_children_data = []
        for child in section_children:
            section_children_data.append(self.build_resume_data(child, all_sections))

        # Get all CSS styles associated with section.
        section_css_data = set()
        section_css = current_section.css.all()
        for css_object in section_css:
            css_classes = css_object.css_classes.strip().split(' ')
            for css_class in css_classes:
                section_css_data.add(css_class)
        section_css_data = ' '.join(list(section_css_data))

        # Organize data value.
        return {
            'section': current_section,
            'section_title': current_section.title,
            'section_description': current_section.description,
            'section_children': section_children_data,
            'section_css': section_css_data,
            'separator_before': current_section.separator_before,
            'separator_after': current_section.separator_after,
            'minimalistic': current_section.minimalistic,
            'display_category': ResumeSection.DISPLAY_CATEGORY_CHOICES[current_section.display_category][1],
        }
